/*
Programa para Arduino Nano y un DS1307 para formar un Reloj típico de mesa, el cual
mostrará la hora y otros datos en un display OLED (Adafruit_SSD1306).

La comunicación entre el Nano y el DS1307 se realiza por el bus I2C, con los siguientes
pines:

SEÑAL NANO  DS1307  EEPROM  OLED
SCL   A5    PIN 6   PIN 6   
SDA   A4    PIN 5   PIN 5

El dS1307 tiene el PIN 3 para conectar una batería de 3V que evita la pérdida de datos al
apagar el circuito. También tiene el PIN 7 como salida programable, para encender algún
dispositivo electrónico en un tiempo debidamente programado.

en la eeprom AT27C128, se puede grabar información, en vez de hacerlo en la eeprom del Nano,
de tal manera que éste último no participe en dicha grabación.

Se utiliza un Encoder Rotativo con Push Button integrado, para proporcionar los comandos
necesarios para ajustar, la fecha, la hora, el pulso de acción del PIN 7 del DS1307, etc.

Para este encoder se usan los pines D2 y D3 del Nano, ya que son entradas INT que pueden
programarse con una rutina de interrupción. Además el Push Button se conecta al pin D4

Tomado del link: https://naylampmechatronics.com/blog/52_tutorial-rtc-ds1307-y-eeprom-at24c.html

con modificaciones propias.

Alfredo Segura, Querétaro, México, septiembre de 2024.

*/
#include <stdio.h>
#include <Wire.h>
#include "RTClib.h"
#include <Rotary.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>


RTC_DS1307 rtc;

int segundo = 36;
int minuto = 57;
int hora = 7;
int dia = 2;
int mes = 9;
int anio = 2024;
int diaDeLaSemana = 4;    // dDLS

bool flag = false;
byte encoder = 1;
int PinCLK = 2;       // Pin D2 para el Encoder, giro a la derecha
int PinDT = 3;        // Pin D3 para el Encoder, giro a la izquierda
int PinSW = 4;        // Pin D4 para el Push Button del Encoder
int PinOut = 5;       // Pin D5 como salida para actuar sobre un dispositivo externo
int sum = 0;
int modo = 0;         // Cambiará a 1: "hora", 1: "minuto", 3: "segundo", si se oprime SW Button en Encoder
                      // 4: "dia", 5: "mes" o 6: "DiaDeLaSemana" (dDLS)

DateTime HoraFecha;

Rotary r = Rotary(2, 3);                    // Pin D2 (INT0) y D3 (INT1) del Nano
Adafruit_SSD1306 display(128, 64, &Wire, -1);

ISR(PCINT2_vect) {
  char result = r.process();
  if (result == DIR_CW) set_hora(-1);        // Este '1' o '-1' se convertirá en la variable 'dir'
  else if (result == DIR_CCW) set_hora(1); // en la función set_hora(short dir).
}

void set_hora(short dir) {
  if (encoder == 1) {
    if (dir == 1) {
      sum = 1;
    }
    if (dir == -1) {
      sum = -1;
    }
  }
  flag = true;
  digitalWrite(PinOut, HIGH);
}

void mostrar() {
  display.clearDisplay();    
  display.setTextSize(2);
  display.setTextColor(SSD1306_WHITE);
  display.setCursor(0, 1);
   
  display.setTextSize(2);
  display.setCursor(8, 10);
  display.println(hora);
  display.setCursor(35, 10);
  display.println(":");
  if (modo == 1) {
    display.setCursor(8, 15);
    display.println("__");
  }
  
  display.setTextSize(2);
  display.setCursor(50, 10);      // Columna, Renglón
  display.println(minuto);
  display.setCursor(77, 10);
  display.println(":");
  if (modo == 2) {
    display.setCursor(50, 15);
    display.println("__");
  }
  
  display.setTextSize(2);
  display.setCursor(95, 10);      // Columna, Renglón
  display.println(segundo);
  if (modo == 3) {
    display.setCursor(95, 15);
    display.println("__");
  }

  display.setTextSize(1);
  display.setCursor(15, 45);       // 3, 45
  display.println(dia);
  display.setCursor(35, 45);      // 30, 45
  display.println("/");
  if (modo == 4) {
    display.setCursor(15, 50);     // 3, 50
    display.println("__");
  }

  display.setTextSize(1);
  display.setCursor(55, 45);      // 50, 45
  display.println(mes);
  display.setCursor(70, 45);      // 77, 45
  display.println("/");  
  if (modo == 5) {
    display.setCursor(55, 50);    // 50, 50
    display.println("__");
  }

  display.setTextSize(1);
  display.setCursor(85, 45);      // 95, 45
  display.println(anio);
  if (modo == 6) {
    display.setCursor(85, 50);    // 95, 50
    display.println("__");
  }  
  
  display.display();
}

void setup () {
  //Serial.begin(9600);
  rtc.begin();                                // Inicializamos el RTC
  //rtc.adjust(DateTime(F(__DATE__), F(__TIME__))); // Grabamos la hora y fecha actuales
  
  pinMode(PinOut, OUTPUT);                    // Salida D5 del Nano para señal programable al exterior
  pinMode(PinCLK, INPUT_PULLUP);              // D2
  pinMode(PinDT, INPUT_PULLUP);               // D3
  pinMode(PinSW, INPUT_PULLUP);               // D4

  PCICR |= (1 << PCIE2);
  PCMSK2 |= (1 << PCINT18) | (1 << PCINT19);
  sei();

  Wire.begin();
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);
  display.clearDisplay();
  display.setTextColor(WHITE);
  display.display();
}

void loop () {
    
  HoraFecha = rtc.now();                        //obtenemos la hora y fecha actual
  
  segundo = HoraFecha.second();
  minuto = HoraFecha.minute();
  hora = HoraFecha.hour();
  diaDeLaSemana = HoraFecha.dayOfTheWeek();     //0: Domingo, 1: Lunes, 2: Martes ...
  dia = HoraFecha.day();
  mes = HoraFecha.month();
  anio = HoraFecha.year();
  
  float horas = hora + minuto / 60.0;

  //condicion: >Domingo y  <sabado    y   >6.30pm  y <11.30pm
  if(diaDeLaSemana > 0 && diaDeLaSemana < 7 && horas > 18.5 && horas < 23.5) {
    digitalWrite(PinOut, HIGH);
    //Serial.print("Salida digital D5 = ON");
  }
  else {
    digitalWrite(PinOut, LOW);
    //Serial.print("Salida digital D5 = OFF");
  }

  if (flag == true) {                       // Se giró el Encoder
    switch (modo) {
      case 1:
        hora = hora + sum;
        rtc.adjust(DateTime(anio, mes, dia, hora, minuto, segundo));
        if (hora >= 24) {
          hora = 0;  
        }
        if (hora < 0) {
          hora = 23;
        }
        break;
      case 2:
        minuto = minuto + sum;
        rtc.adjust(DateTime(anio, mes, dia, hora, minuto, segundo));
        if (minuto >= 60) {
          minuto = 0;
        }
        if (minuto < 0) {
          minuto = 59;
        }
        break;
      case 3:
        segundo = segundo + sum;
        rtc.adjust(DateTime(anio, mes, dia, hora, minuto, segundo));
        if (segundo >= 60) {
          segundo = 0;
        }
        if (segundo < 0) {
          segundo = 59;
        }
        break;
      case 4:
        dia = dia + sum;
        rtc.adjust(DateTime(anio, mes, dia, hora, minuto, segundo));
        if (dia > 31) {
          dia = 1;
        }
        if (dia < 1) {
          dia = 31;  
        }
        break;
      case 5:
        mes = mes + sum;
        rtc.adjust(DateTime(anio, mes, dia, hora, minuto, segundo));
        if (mes > 12) {
          mes = 1;  
        }
        if (mes < 1) {
          mes = 12;
        }
        break;
      case 6:
        anio = anio + sum;
        rtc.adjust(DateTime(anio, mes, dia, hora, minuto, segundo));
        if (anio > 2099) {
          anio = 2000;
        }
        if (anio < 2000) {
          anio = 2099;
        }
        break;
    
    modo = 0;
    sum = 0;    
    flag = false;
    }
  }

  if (digitalRead(PinSW) == 0) {      // SW Button del Encoder oprimido
    delay(100);                        // Anti-rebotes
    modo = modo + 1;
    if (modo > 6) {
      modo = 0;  
    }
  }
  
  //Enviamos por el puerto serie la hora y fecha.
  /*
  Serial.print("   hora = ");
  Serial.print(hora);
  Serial.print(":");
  Serial.print(minuto);
  Serial.print(":");
  Serial.print(segundo);
  Serial.print("  dia de La semana = ");
  Serial.print(HoraFecha.dayOfTheWeek());
  Serial.print("  sum = ");
  Serial.print(sum);
  Serial.print("  modo = ");
  Serial.print(modo);
  Serial.println();
  */
  sum = 0;
  mostrar();
  digitalWrite(PinOut, LOW);
  delay(1000);
    
}

    
